import socket
import requests
import argparse
import struct

MAX_RECV_BYTES = 512*1024
RECV_BUF_SIZE = 1024*1024
SEND_BUF_SIZE = 1024*1024

def get_public_ip():
    # Define the URL for the external service that returns the public IP address
    url = "https://api.ipify.org?format=json"
    # Make a GET request to the URL
    response = requests.get(url)
    # Check the response status code
    if response.status_code == 200:
        # Parse the JSON response
        data = response.json()
        # Get the public IP address from the response
        public_ip = data.get("ip")
        return public_ip
    else:
        # Handle the case where the request failed
        print(f"Failed to get public IP. Status code: {response.status_code}")
        return None

def set_socket_buffer_sizes(sock, rcv_buf_size, snd_buf_size):
    """
    Set the receive and send buffer sizes for the given socket.

    Args:
        sock: The socket object.
        rcv_buf_size: The desired receive buffer size in bytes.
        snd_buf_size: The desired send buffer size in bytes.
    """
    # Set the receive buffer size
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, struct.pack("I", rcv_buf_size))

    # Set the send buffer size
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, struct.pack("I", snd_buf_size))

    # Get and print the buffer sizes set on the socket
    actual_rcv_buf = sock.getsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF)
    actual_snd_buf = sock.getsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF)

    print(f"Receive buffer size set to: {actual_rcv_buf} bytes")
    print(f"Send buffer size set to: {actual_snd_buf} bytes")

def start_tcp_server(port):
    # Create a TCP socket
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    # Set the buffer sizes (e.g., 1 MB)
    set_socket_buffer_sizes(server_socket, RECV_BUF_SIZE, SEND_BUF_SIZE)  # 1 MB for receive and send buffer sizes

    # Bind the socket to the IP address and port
    server_socket.bind(("0.0.0.0", port))
    # Listen for incoming connections
    server_socket.listen(1)
    print(f"TCP server started. Listening on 0.0.0.0:{port}")
    tot_bytes = 0
    while True:
        # Accept an incoming connection
        conn, addr = server_socket.accept()
        print(f"TCP connection established with {addr}")

        try:
            # Echo data back to the client
            while True:
                data = conn.recv(MAX_RECV_BYTES)
                if not data:
                    # If no data received, break the loop (client may have disconnected)
                    break
                tot_bytes += len(data)
                print(f" TCP Received data {len(data)} bytes from {addr}, total data received: {tot_bytes} bytes ", end="")
                #for byte in data:
                #    print(f"0x{byte:02x} ", end="")
                print()  # New line after printing all bytes
                # Echo the received data back to the client
                conn.sendall(data)
        except Exception as e:
            print(f"An error occurred: {e}")
        finally:
            # Close the connection
            conn.close()
            print(f"TCP connection with {addr} closed")

def start_udp_server(port):
    # Create a UDP socket
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    # Set the buffer sizes (e.g., 1 MB)
    set_socket_buffer_sizes(server_socket, RECV_BUF_SIZE, SEND_BUF_SIZE)  # 1 MB for receive and send buffer sizes

    # Bind the socket to the IP address and port
    server_socket.bind(("0.0.0.0", port))
    print(f"UDP server started. Listening on 0.0.0.0:{port}")
    tot_bytes = 0
    while True:
        # Receive data from the client
        data, addr = server_socket.recvfrom(MAX_RECV_BYTES)
        tot_bytes += len(data)
        print(f"UDP data {len(data)} bytes received from {addr}, total data received: {tot_bytes} bytes ", end="")
        #for byte in data:
        #    print(f"0x{byte:02x} ", end="")
        print()  # New line after printing all bytes
        # Echo the received data back to the client
        server_socket.sendto(data, addr)

def main():
    # Create an argument parser
    parser = argparse.ArgumentParser(description="Start a TCP or UDP server.")
    # Add argument for the server mode (TCP or UDP)
    parser.add_argument("mode", choices=["tcp", "udp"], help="Server mode: tcp or udp")
    # Add argument for the port number
    parser.add_argument("-p", "--port", type=int, default=12345, help="Port number (default: 12345)")
    # Parse the command-line arguments
    args = parser.parse_args()

    # Get the public IP address
    public_ip = get_public_ip()
    if public_ip:
        print(f"Your public IP address is: {public_ip}")

    # Start the server based on the selected mode
    if args.mode == "tcp":
        start_tcp_server(args.port)
    elif args.mode == "udp":
        start_udp_server(args.port)

if __name__ == "__main__":
    main()
